from dapiclient.client import DAPIClient
from thumbor_dash.error_handlers.sentry import ErrorHandler
from thumbor_dash.error_handlers import *

import cbor2
import base58



def getDocuments(handler, data, seed_ip = None, mn_ip = None):
    client = DAPIClient(seed_ip=seed_ip, mn_ip = mn_ip)
    error_handler = ErrorHandler(handler.context.config)

    docs = client.getDocuments(
        data['contract_id'],
        data['document_type'],
        data['where'],
        limit=2, # Only one document
        )

    documents = []
    
    for  doc in docs:
        documents.append(cbor2.loads(doc))
    
    return documents[0] # Return the only document in the list


def getIdentity(handler, ownerId, seed_ip = None, mn_ip = None):
    client = DAPIClient(seed_ip=seed_ip, mn_ip = mn_ip)
    error_handler = ErrorHandler(handler.context.config)

    try:
        identity = client.getIdentity(ownerId)
    except Exception as e:
        error_handler.handle_error(handler.context, handler, UnknownUserError)
        return

    else:   
        return identity # Return the only document in the list






import re
import hashlib

from six import b
from libthumbor.url import Url 

AVAILABLE_HALIGN = ["left", "center", "right"]
AVAILABLE_VALIGN = ["top", "middle", "bottom"]


def calculate_width_and_height(url_parts, options):
     """Appends width and height information to url"""
     width = options.get("width", 0)
     has_width = width
     height = options.get("height", 0)
     has_height = height

     flip = options.get("flip", False)
     flop = options.get("flop", False)

     if flip:
          width = width * -1
     if flop:
          height = height * -1

     if not has_width and not has_height:
          if flip:
               width = "-0"
          if flop:
               height = "-0"

     if width or height:
          url_parts.append("%sx%s" % (width, height))


def url_for(**options):
    """Returns the url for the specified options"""

    url_parts = get_url_parts(**options)
    image_hash = hashlib.md5(b(options["image_url"])).hexdigest()
    url_parts.append(image_hash)

    return "/".join(url_parts)


def unsafe_url(**options):
    """Returns the unsafe url with the specified options"""

    return "unsafe/%s" % plain_image_url(**options)


def plain_image_url(**options):
    url_parts = get_url_parts(**options)
    url_parts.append(options["image_url"])

    return "/".join(url_parts)


def get_url_parts(**options):
     if "image_url" not in options:
          raise ValueError("The image_url argument is mandatory.")

     url_parts = []

     if options.get("meta", False):
          url_parts.append("meta")

     trim = options.get("trim", None)
     if trim:
          bits = ["trim"]
          if not isinstance(trim, bool):
               bits.append(trim[0] if trim[0] else "")
               if trim[1]:
                    bits.append(str(trim[1]))
          url_parts.append(":".join(bits))

     crop = options.get("crop", None)
     if crop:
          crop_left = crop[0][0]
          crop_top = crop[0][1]
          crop_right = crop[1][0]
          crop_bottom = crop[1][1]

          if crop_left > 0 or crop_top > 0 or crop_bottom > 0 or crop_right > 0:
               url_parts.append(
                    "%sx%s:%sx%s" % (crop_left, crop_top, crop_right, crop_bottom)
                    )

     calculate_fit_in(options, url_parts)
     calculate_width_and_height(url_parts, options)

     halign = options.get("halign", "center")
     valign = options.get("valign", "middle")

     if halign not in AVAILABLE_HALIGN:
          raise ValueError(
               'Only "left", "center" and "right" are'
               + " valid values for horizontal alignment."
               )
     if valign not in AVAILABLE_VALIGN:
          raise ValueError(
               'Only "top", "middle" and "bottom" are'
               + " valid values for vertical alignment."
               )

     if halign != "center":
          url_parts.append(halign)
     if valign != "middle":
          url_parts.append(valign)

     if options.get("smart", False):
          url_parts.append("smart")
     
     if options.get("dashauth", False):
          dashauth_string = ["dashauth"]
          for dashauth_param in options["dashauth"]:
               dashauth_string.append(dashauth_param)
          url_parts.append(":".join(dashauth_string))

     if options.get("filters", False):
          filters_string = ["filters"]
          for filter_value in options["filters"]:
               filters_string.append(filter_value)
          url_parts.append(":".join(filters_string))
     
     return url_parts


def calculate_fit_in(options, url_parts):
     fit_in = False
     full_fit_in = False

     if options.get("fit_in", None):
          fit_in = True
          url_parts.append("fit-in")

     if options.get("full_fit_in", None):
         full_fit_in = True
         url_parts.append("full-fit-in")

     if options.get("adaptive_fit_in", None):
         fit_in = True
         url_parts.append("adaptive-fit-in")

     if options.get("adaptive_full_fit_in", None):
         full_fit_in = True
         url_parts.append("adaptive-full-fit-in")

     if (fit_in or full_fit_in) and not (
         options.get("width", None) or options.get("height", None)
     ):
         raise ValueError(
              "When using fit-in or full-fit-in, you must specify width and/or height."
              )

class ThumborDashUrl(Url):

    dashauth = r"(?:dashauth:(?P<dashauth>.+?\))/)?"

    @classmethod
    def regex(cls, has_unsafe_or_hash=True):
       reg = ["/?"]

       if has_unsafe_or_hash:
            reg.append(cls.unsafe_or_hash)
       reg.append(cls.debug)
       reg.append(cls.meta)
       reg.append(cls.trim)
       reg.append(cls.crop)
       reg.append(cls.fit_in)
       reg.append(cls.dimensions)
       reg.append(cls.halign)
       reg.append(cls.valign)
       reg.append(cls.smart)
       reg.append(cls.dashauth)
       reg.append(cls.filters)
       reg.append(cls.image)
       
       return "".join(reg)




import mimetypes
import random
from io import BytesIO
from os.path import dirname, join, realpath
from urllib.parse import urlencode

import mock
from PIL import Image
from SSIM_PIL import compare_ssim
from tornado.testing import AsyncHTTPTestCase

from thumbor_dash.app import ThumborDashServiceApp
from thumbor.config import Config
from thumbor_dash.context import *
from thumbor.engines.pil import Engine as PilEngine
from thumbor.importer import Importer
from thumbor.transformer import Transformer


def get_ssim(actual, expected):
    if actual.size[0] != expected.size[0] or actual.size[1] != expected.size[1]:
        raise RuntimeError(
            "Can't calculate SSIM for images of different sizes "
            "(one is %dx%d, the other %dx%d)."
            % (actual.size[0], actual.size[1], expected.size[0], expected.size[1],)
        )

    return compare_ssim(actual, expected)


def encode_multipart_formdata(fields, files):
    boundary = b"thumborUploadFormBoundary"
    crlf = b"\r\n"
    lines = []
    for key, value in fields.items():
        lines.append(b"--" + boundary)
        lines.append(b'Content-Disposition: form-data; name="%s"' % key.encode())
        lines.append(b"")
        lines.append(value)
    for (key, filename, value) in files:
        lines.append(b"--" + boundary)
        lines.append(
            b'Content-Disposition: form-data; name="%s"; filename="%s"'
            % (key.encode(), filename.encode())
        )
        lines.append(
            b"Content-Type: %s" % mimetypes.guess_type(filename)[0].encode()
            or b"application/octet-stream"
        )
        lines.append(b"")
        lines.append(value)
    lines.append(b"")
    lines.append(b"")
    lines.append(b"--" + boundary + b"--")
    body = crlf.join(lines)
    content_type = b"multipart/form-data; boundary=%s" % boundary
    return content_type, body



class TestCase(AsyncHTTPTestCase):
    _multiprocess_can_split_ = True

    def get_app(self):
        self.context = self.get_context()
        return ThumborDashServiceApp(self.context)

    def get_config(self):  # Meant to be overriden pylint: disable=no-self-use
        return Config()

    def get_server(self):  # Meant to be overriden pylint: disable=no-self-use
        return None

    def get_importer(self):
        importer = Importer(self.config)
        importer.import_modules()
        return importer

    def get_request_handler(self,):  # Meant to be overriden pylint: disable=no-self-use
        return None

    def get_context(self):
        self.config = (  # This is a test case pylint: disable=attribute-defined-outside-init
            self.get_config()
        )
        self.server = (  # This is a test case pylint: disable=attribute-defined-outside-init
            self.get_server()
        )
        self.importer = (  # This is a test case pylint: disable=attribute-defined-outside-init
            self.get_importer()
        )
        self.request_handler = (  # This is a test case pylint: disable=attribute-defined-outside-init
            self.get_request_handler()
        )
        self.importer.import_modules()
        return ThumborDashContext(self.server, self.config, self.importer, self.request_handler)

    async def async_fetch(self, path, method="GET", body=None, headers=None):
        return await self.http_client.fetch(
            self.get_url(path),
            method=method,
            body=body,
            headers=headers,
            allow_nonstandard_methods=True,
            raise_error=False,
        )

    async def async_get(self, path, headers=None):
        return await self.async_fetch(
            path, method="GET", body=urlencode({}, doseq=True), headers=headers
        )

    async def async_post(self, path, headers, body):
        return await self.async_fetch(path, method="POST", body=body, headers=headers,)

    async def async_put(self, path, headers, body):
        return await self.async_fetch(path, method="PUT", body=body, headers=headers,)

    async def async_delete(self, path, headers):
        return await self.async_fetch(
            path, method="DELETE", body=urlencode({}, doseq=True), headers=headers,
        )

    async def async_post_files(self, path, data=None, files=None):
        if data is None:
            data = {}
        if files is None:
            files = []
        multipart_data = encode_multipart_formdata(data, files)

        return await self.async_fetch(
            path,
            method="POST",
            body=multipart_data[1],
            headers={"Content-Type": multipart_data[0]},
        )

class FilterTestCase(TestCase):
    _multiprocess_can_split_ = True

    def setUp(self):
        super().setUp()
        self.context = {}

    def get_filter(self, filter_name, params_string="", config_context=None):
        config = Config(
            FILTERS=[filter_name],
            LOADER="thumbor.loaders.file_loader",
            FILE_LOADER_ROOT_PATH=self.get_fixture_root_path(),
        )
        importer = Importer(config)
        importer.import_modules()

        req = ThumborDashRequestParameters()

        context = ThumborDashContext(config=config, importer=importer)
        context.request = req
        context.request.engine = context.modules.engine

        if config_context is not None:
            config_context(context)

        self.context = context

        fltr = importer.filters[0]
        fltr.pre_compile()

        context.transformer = Transformer(context)

        return fltr(params_string, context=context)

    @staticmethod
    def get_fixture_root_path():
        return join(dirname(realpath(__file__)), "fixtures", "filters")

    def get_fixture_path(self, name):
        return f"{self.get_fixture_root_path()}/{name}"

    def get_fixture(self, name, mode="RGB"):
        image = Image.open(self.get_fixture_path(name))
        return image.convert(mode)

    async def get_filtered(
        self, source_image, filter_name, params_string, config_context=None, mode="RGB",
    ):
        fltr = self.get_filter(filter_name, params_string, config_context)
        image = Image.open(self.get_fixture_path(source_image))
        img_buffer = BytesIO()

        if params_string == "quality(10)":
            image.save(img_buffer, "JPEG", quality=10)
            fltr.engine.load(img_buffer.getvalue(), ".jpg")
        else:
            image.save(img_buffer, "PNG", quality=100)
            fltr.engine.load(img_buffer.getvalue(), ".png")

        fltr.context.transformer.img_operation_worker()

        await fltr.run()

        fltr.engine.image = fltr.engine.image.convert(mode)

        return fltr.engine.image

    @staticmethod
    def get_ssim(actual, expected):
        return get_ssim(actual, expected)

    def debug(self, image):  # pylint: disable=arguments-differ
        image = Image.fromarray(image)
        path = "/tmp/debug_image_%s.jpg" % random.randint(1, 10000)
        image.save(path, "JPEG")
        print("The debug image was in %s." % path)

    @staticmethod
    def debug_size(image):
        loaded = Image.fromarray(image)
        print(
            "Image dimensions are %dx%d (shape is %s)"
            % (loaded.size[0], loaded.size[1], image.shape)
        )


class DetectorTestCase(TestCase):
    _multiprocess_can_split_ = True

    def setUp(self):
        super().setUp()
        self.context.request = mock.Mock(focal_points=[])
        self.engine = PilEngine(self.context)
        self.context.modules.engine = self.engine


from thumbor_dash.storages.request_storage import RequestStorage
from json import dumps, loads
from datetime import datetime
from thumbor_dash.utils import datetimeToMillisecondsSinceEpoch



async def verifyUserAccessStatus(requester_id, config):
    ''' Checks requester's ban status '''
    REQUEST_TIME_LIMIT = config.REQUEST_TIME_LIMIT * 60 * 1000 #convert minutes to milliseconds
    USAGE_VIOLATION_LIMIT = config.USAGE_VIOLATION_LIMIT 
    BAN_DURATION = config.BAN_DURATION * 60 * 1000 #convert minutes to milliseconds

    storage = RequestStorage()
    storage_get_result = await storage.get(requester_id=requester_id)
 
    if storage_get_result is None:
        storage_put_result = await update_requester_data(requester_id=requester_id, storage=storage, is_banned=False, last_accessed=datetimeToMillisecondsSinceEpoch(datetime.now()), usage_violation_count=0, next_access=0)
        return True
    else:
        requester_storage_data = loads(storage_get_result)
  
        is_banned = requester_storage_data['is_banned']
        last_accessed = requester_storage_data['last_accessed']
        usage_violation_count = requester_storage_data['usage_violation_count']
        next_access = requester_storage_data['next_access']

        current_time = datetimeToMillisecondsSinceEpoch(datetime.now())
        
        if ((current_time - last_accessed) < REQUEST_TIME_LIMIT) and is_banned==False:
            usage_violation_count = usage_violation_count + 1

        if (usage_violation_count > USAGE_VIOLATION_LIMIT) and is_banned==False:
            is_banned = True
            next_access = current_time + BAN_DURATION

        if is_banned:
            if next_access <= current_time:
                is_banned = False
                usage_violation_count = 0
                storage_put_result = await update_requester_data(requester_id=requester_id, storage=storage, is_banned=is_banned, last_accessed=current_time, usage_violation_count=usage_violation_count, next_access=next_access)
                return True
            else:   
                storage_put_result = await update_requester_data(requester_id=requester_id, storage=storage, is_banned=is_banned, last_accessed=current_time, usage_violation_count=usage_violation_count, next_access=next_access)
                return False
            
        storage_put_result = await update_requester_data(requester_id=requester_id, storage=storage, is_banned=is_banned, last_accessed=current_time, usage_violation_count=usage_violation_count, next_access=next_access)
        return True
        

async def update_requester_data(requester_id, storage, is_banned, last_accessed, usage_violation_count, next_access):
    requester_data = {
        'is_banned': is_banned,
        'last_accessed': last_accessed,
        'usage_violation_count': usage_violation_count,
        'next_access': next_access # next time a user gets access after a ban
        }
    file_bytes = dumps(requester_data).encode('utf-8')
    storage_put_result = await storage.put(requester_id=requester_id, file_bytes=file_bytes)   





"""Encrypted URLs for thumbor encryption."""

import base64
import hashlib
import hmac

from libthumbor.crypto import CryptoURL
from six import PY3, b, text_type
from thumbor_dash.url import plain_image_url, unsafe_url

class ThumborDashCryptoURL(CryptoURL):
    """Class responsible for generating encrypted URLs for thumbor_dash"""
    
    def __init__(self, key):
        """
        Initializes the encryptor with the proper key
        :param key: secret key to use for hashing.
        """

        if isinstance(key, text_type):
            key = str(key)
        self.key = key
        self.hmac = hmac.new(b(key), digestmod=hashlib.sha256)

    def generate_new(self, options):
        url = plain_image_url(**options)
        _hmac = self.hmac.copy()
        _hmac.update(text_type(url).encode("utf-8"))
        signature = base64.urlsafe_b64encode(_hmac.digest())

        if PY3:
            signature = signature.decode("ascii")
        return "/%s/%s" % (signature, url)

    def generate(self, **options):
        """Generates an encrypted URL with the specified options"""

        if options.get("unsafe", False):
            return unsafe_url(**options)

        return self.generate_new(options)


import sys
from thumbor_dash.app import ThumborDashServiceApp

import tornado.ioloop
from tornado.httpserver import HTTPServer
import warnings
from PIL import Image

from shutil import which
from thumbor.console import get_server_parameters
from thumbor.signal_handler import setup_signal_handler
from thumbor.server import run_server, get_importer, get_config, configure_log, get_context, get_as_integer

def get_application(context):
    return ThumborDashServiceApp(context)

def validate_config(config, server_parameters):
    if server_parameters.security_key is None:
        server_parameters.security_key = config.SECURITY_KEY

    if not isinstance(server_parameters.security_key, (bytes, str)):
        raise RuntimeError(
            "No security key was found for this instance of thumbor. "
            + "Please provide one using the conf file or a security key file."
        )

    if config.ENGINE or config.USE_GIFSICLE_ENGINE:
        # Error on Image.open when image pixel count is above MAX_IMAGE_PIXELS
        warnings.simplefilter("error", Image.DecompressionBombWarning)

    if config.USE_GIFSICLE_ENGINE:
        server_parameters.gifsicle_path = which("gifsicle")
        if server_parameters.gifsicle_path is None:
            raise RuntimeError(
                "If using USE_GIFSICLE_ENGINE configuration to True,"
                " the `gifsicle` binary must be in the PATH "
                "and must be an executable."
            )

    # Validate thumbor_dash user moderation settings
    if config.get("REQUEST_TIME_LIMIT") is None:
        raise RuntimeError(
            "Thumbor_Dash Error: Request time limit is not set. Please provide a time value for REQUEST_TIME_LIMIT in .conf file"
        )
 
    if config.get("USAGE_VIOLATION_LIMIT") is None:
        raise RuntimeError(
            "Thumbor_Dash Error: Usage violation limit is not set. Please provide a value for USAGE_VIOLATION_LIMIT in .conf file"
        )   

    if config.get("BAN_DURATION") is None:
        raise RuntimeError(
            "Thumbor_Dash Error: Ban duration is not set. Please provide a time value for BAN_DURATION in .conf file"
        )   

def main(arguments=None):
    """Runs thumbor server with the specified arguments."""
    if arguments is None:
        arguments = sys.argv[1:]

    server_parameters = get_server_parameters(arguments)
    config = get_config(
        server_parameters.config_path, server_parameters.use_environment
    )

    configure_log(config, server_parameters.log_level.upper())

    validate_config(config, server_parameters)

    importer = get_importer(config)

    with get_context(server_parameters, config, importer) as context:
        application = get_application(context)
        server = run_server(application, context)
        setup_signal_handler(server, config)
        tornado.ioloop.IOLoop.instance().start()


if __name__ == "__main__":
    main(sys.argv[1:])


from os.path import abspath, exists

from thumbor.filters import FiltersFactory
from thumbor.metrics.logger_metrics import Metrics
from thumbor.threadpool import ThreadPool
from thumbor.context import *


class ThumborDashContext(Context):
    """
    Class responsible for containing:
    * Server Configuration Parameters (port, ip, key, etc);
    * Configurations read from config file (or defaults);
    * Importer with imported modules (engine, filters, detectors, etc);
    * Request Parameters (width, height, smart, meta, etc).
    Each instance of this class MUST be unique per request.
    This class should not be cached in the server.
    """
    def __init__(self, server=None, config=None, importer=None, request_handler=None):
        self.server = server
        self.config = config
        if importer:
            self.modules = ThumborDashContextImporter(self, importer)
            if importer.metrics:
                self.metrics = importer.metrics(config)
            else:
                self.metrics = Metrics(config)
        else:
            self.modules = None
            self.metrics = Metrics(config)

        self.app_class = "thumbor_dash.app.ThumborDashServiceApp"

        if hasattr(self.config, "APP_CLASS"):
            self.app_class = self.config.APP_CLASS

        if (
            hasattr(self.server, "app_class")
            and self.server.app_class != "thumbor_dash.app.ThumborDashServiceApp"
        ):
            self.app_class = self.server.app_class

        self.filters_factory = FiltersFactory(
            self.modules.filters if self.modules else []
        )
        self.request_handler = request_handler
        self.thread_pool = ThreadPool.instance(
            getattr(config, "ENGINE_THREADPOOL_SIZE", 0)
        )
        self.headers = {}



class ThumborDashContextImporter(ContextImporter):
    def __init__(self, context, importer):
        super().__init__(context, importer)




class ThumborDashRequestParameters(RequestParameters):
    def __init__(
        self,
        debug=False,
        meta=False,
        trim=None,
        crop_left=None,
        crop_top=None,
        crop_right=None,
        crop_bottom=None,
        crop=None,
        adaptive=False,
        full=False,
        fit_in=False,
        stretch=False,
        width=0,
        height=0,
        horizontal_flip=False,
        vertical_flip=False,
        halign="center",
        valign="middle",
        dashauth = None,
        filters=None,
        smart=False,
        quality=80,
        image=None,
        url=None,
        extension=None,  
        buffer=None,  
        focal_points=None,
        unsafe=False,
        hash=None,  
        accepts_webp=False,
        request=None,
        max_age=None,
        auto_png_to_jpg=None,
    ):
        self.debug = bool(debug)
        self.meta = bool(meta)
        self.trim = trim
        if trim is not None:
            trim_parts = trim.split(":")
            self.trim_pos = trim_parts[1] if len(trim_parts) > 1 else "top-left"
            self.trim_tolerance = int(trim_parts[2]) if len(trim_parts) > 2 else 0

        if crop is not None:
            self.crop = {k: self.int_or_0(v) for k, v in crop.items()}
        else:
            self.crop = {
                "left": self.int_or_0(crop_left),
                "right": self.int_or_0(crop_right),
                "top": self.int_or_0(crop_top),
                "bottom": self.int_or_0(crop_bottom),
            }

        self.should_crop = (
            self.crop["left"] > 0
            or self.crop["top"] > 0
            or self.crop["right"] > 0
            or self.crop["bottom"] > 0
        )

        self.adaptive = bool(adaptive)
        self.full = bool(full)
        self.fit_in = bool(fit_in)
        self.stretch = bool(stretch)

        self.width = "orig" if width == "orig" else self.int_or_0(width)
        self.height = "orig" if height == "orig" else self.int_or_0(height)
        self.horizontal_flip = bool(horizontal_flip)
        self.vertical_flip = bool(vertical_flip)
        self.halign = halign or "center"
        self.valign = valign or "middle"
        self.smart = bool(smart)

        if filters is None:
            filters = []

        self.filters = filters
        self.dashauth = dashauth
        self.image_url = image
        self.url = url
        self.detection_error = None
        self.quality = quality
        self.buffer = None

        if focal_points is None:
            focal_points = []

        self.focal_points = focal_points
        self.hash = hash
        self.prevent_result_storage = False
        self.unsafe = unsafe == "unsafe" or unsafe is True
        self.format = None
        self.accepts_webp = accepts_webp
        self.max_bytes = None
        self.max_age = max_age
        self.auto_png_to_jpg = auto_png_to_jpg

        if request:
            self.url = request.path
            self.accepts_webp = "image/webp" in request.headers.get("Accept", "")

    @staticmethod
    def int_or_0(value):
        return 0 if value is None else int(value)


import optparse  
import sys
from urllib.parse import quote

from thumbor_dash.crypto import ThumborDashCryptoURL

from thumbor import __version__
from thumbor.config import Config


def get_parser():
    parser = optparse.OptionParser(
        usage="thumbor_dash-url [options] imageurl or type thumbor_dash-url -h (--help) for help",
        description=__doc__,
        version=__version__,
    )

    parser.add_option(
        "-l",
        "--key_file",
        dest="key_file",
        default=None,
        help="The file to read the security key from [default: %default].",
    )
    parser.add_option(
        "-k",
        "--key",
        dest="key",
        default=None,
        help="The security key to encrypt the url with [default: %default].",
    )
    parser.add_option(
        "-w",
        "--width",
        dest="width",
        type="int",
        default=0,
        help="The target width for the image [default: %default].",
    )
    parser.add_option(
        "-e",
        "--height",
        dest="height",
        type="int",
        default=0,
        help="The target height for the image [default: %default].",
    )
    parser.add_option(
        "-n",
        "--fitin",
        dest="fitin",
        action="store_true",
        default=False,
        help="Indicates that fit-in resizing should be performed.",
    )
    parser.add_option(
        "-m",
        "--meta",
        dest="meta",
        action="store_true",
        default=False,
        help="Indicates that meta information should be retrieved.",
    )
    parser.add_option(
        "",
        "--adaptive",
        action="store_true",
        dest="adaptive",
        default=False,
        help="Indicates that adaptive fit-in cropping should be used.",
    )
    parser.add_option(
        "",
        "--full",
        action="store_true",
        dest="full",
        default=False,
        help="Indicates that fit-full cropping should be used.",
    )
    parser.add_option(
        "-s",
        "--smart",
        action="store_true",
        dest="smart",
        default=False,
        help="Indicates that smart cropping should be used.",
    )
    parser.add_option(
        "-t",
        "--trim",
        action="store_true",
        default=False,
        help="Indicate that surrounding whitespace should be trimmed.",
    )
    parser.add_option(
        "-f",
        "--horizontal-flip",
        action="store_true",
        dest="horizontal_flip",
        default=False,
        help="Indicates that the image should be horizontally flipped.",
    )
    parser.add_option(
        "-v",
        "--vertical-flip",
        action="store_true",
        dest="vertical_flip",
        default=False,
        help="Indicates that the image should be vertically flipped.",
    )
    parser.add_option(
        "-a",
        "--halign",
        dest="halign",
        default="center",
        help="The horizontal alignment to use for cropping [default: %default].",
    )
    parser.add_option(
        "-i",
        "--valign",
        dest="valign",
        default="middle",
        help="The vertical alignment to use for cropping [default: %default].",
    )
    parser.add_option(
        "-d",
        "--dashauth",
        dest="dashauth",
        action="append",
        help="Dashauth parameters, e.g requester(<requesterId>).",
    )
    parser.add_option(
        "",
        "--filters",
        dest="filters",
        action="append",
        help="Filters to be applied to the image, e.g. brightness(10).",
    )

    parser.add_option(
        "-c",
        "--crop",
        dest="crop",
        default=None,
        help="The coordinates of the points to manual cropping in the format leftxtop:rightxbottom "
        "(100x200:400x500) [default: %default].",
    )

    return parser


def get_options(arguments):
    if arguments is None:
        arguments = sys.argv[1:]

    parser = get_parser()

    (parsed_options, arguments) = parser.parse_args(arguments)

    if not arguments:
        sys.stdout.write(
            "Error: The image argument is mandatory. For more information type thumbor_dash-url -h\n"
        )
        return None, None

    return parsed_options, arguments


def get_thumbor_params(image_url, params, config):
    if params.key_file:
        with open(params.key_file) as key_file:
            security_key = key_file.read().strip()
    else:
        security_key = config.SECURITY_KEY if not params.key else params.key

    crop_left = crop_top = crop_right = crop_bottom = 0
    if params.crop:
        crops = params.crop.split(":")
        crop_left, crop_top = crops[0].split("x")
        crop_right, crop_bottom = crops[1].split("x")

    options = {
        "width": params.width,
        "height": params.height,
        "smart": params.smart,
        "meta": params.meta,
        "horizontal_flip": params.horizontal_flip,
        "vertical_flip": params.vertical_flip,
        "halign": params.halign,
        "valign": params.valign,
        "trim": params.trim,
        "crop_left": crop_left,
        "crop_top": crop_top,
        "crop_right": crop_right,
        "crop_bottom": crop_bottom,
        "dashauth": params.dashauth,
        "filters": params.filters,
        "image_url": image_url,
        "fit_in": False,
        "full_fit_in": False,
        "adaptive_fit_in": False,
        "adaptive_full_fit_in": False,
    }

    if params.fitin and params.full and params.adaptive:
        options["adaptive_full_fit_in"] = True
    elif params.fitin and params.full:
        options["full_fit_in"] = True
    elif params.fitin and params.adaptive:
        options["adaptive_fit_in"] = True
    elif params.fitin:
        options["fit_in"] = True

    return security_key, options


def main(arguments=None):
    """Converts a given url with the specified arguments."""

    parsed_options, arguments = get_options(arguments)

    image_url = arguments[0]
    image_url = quote(image_url)

    try:
        config = Config.load(None)
    except Exception:  # pylint: disable=broad-except
        config = None

    if not parsed_options.key and not config:
        sys.stdout.write(
            "Error: The -k or --key argument is mandatory."
            " For more information type thumbor_dash-url -h\n"
        )
        return None

    security_key, thumbor_params = get_thumbor_params(image_url, parsed_options, config)

    crypto = ThumborDashCryptoURL(key=security_key)
    url = crypto.generate(**thumbor_params)
    sys.stdout.write("URL:\n")
    sys.stdout.write("%s\n" % url)

    return url


if __name__ == "__main__":
    main(sys.argv[1:])



def verifyImageSize(thumbnail_width, thumbnail_height, MIN_WIDTH, MIN_HEIGHT, MAX_WIDTH, MAX_HEIGHT):
    ''' Checks whether the original image size is within allowed bounds'''
    if thumbnail_width >= MIN_WIDTH and thumbnail_width <= MAX_WIDTH and thumbnail_height >= MIN_HEIGHT and thumbnail_height <= MAX_HEIGHT:
        return True
    else:
        return False 


from typing import Any, cast

from thumbor.handler_lists import HandlerList
from thumbor_dash.handlers.image_resource import ThumborDashImageResourceHandler
from thumbor.handlers.upload import ImageUploadHandler


def get_handlers(context: Any) -> HandlerList:
    is_upload_enabled = cast(bool, context.config.UPLOAD_ENABLED)
    if not is_upload_enabled:
        return []

    return [
        (r"/image", ImageUploadHandler, {"context": context}),
        (r"/image/(.*)", ThumborDashImageResourceHandler, {"context": context}),
    ]


import hashlib
import json
import os
from datetime import datetime
from json import dumps, loads
from os.path import dirname, exists, getmtime, splitext
from shutil import move
from typing import ContextManager
from uuid import uuid4
from thumbor import storages
from os.path import expanduser,  join

from thumbor.utils import logger
from thumbor.context import Context

home = expanduser("~")
REQUEST_STORAGE_ROOT_PATH = join(home, 'thumbor', 'request_storage')
STORAGE_EXPIRATION_SECONDS = 60 * 60 * 24  # one day

class RequestStorage:
    
    async def put(self, requester_id, file_bytes):
        file_abspath = self.path_on_filesystem(requester_id)
        temp_abspath = "%s.%s" % (file_abspath, str(uuid4()).replace("-", ""))
        file_dir_abspath = dirname(file_abspath)

        logger.debug("creating tempfile for %s in %s...", requester_id, temp_abspath)

        self.ensure_dir(file_dir_abspath)

        with open(temp_abspath, "wb") as _file:
            _file.write(file_bytes)

        logger.debug("moving tempfile %s to %s...", temp_abspath, file_abspath)
        move(temp_abspath, file_abspath)

        return requester_id   
    

    def path_on_filesystem(self, requester_id):
        digest = hashlib.sha1(requester_id.encode("utf-8")).hexdigest()
        return "%s/%s/%s" % (
            REQUEST_STORAGE_ROOT_PATH.rstrip("/"),
            digest[:2],
            digest[2:],
        )


    async def get(self, requester_id):
        abs_path = self.path_on_filesystem(requester_id)

        resource_available = await self.exists(requester_id, path_on_filesystem=abs_path)
        if not resource_available:
            return None

        with open(self.path_on_filesystem(requester_id), "rb") as source_file:
            return source_file.read()

    async def exists(
        self, path, path_on_filesystem=None
    ):
        if path_on_filesystem is None:
            path_on_filesystem = self.path_on_filesystem(path)
        return os.path.exists(path_on_filesystem) and not self.__is_expired(
            path_on_filesystem
        )

    async def remove(self, path):
        n_path = self.path_on_filesystem(path)
        return os.remove(n_path)

    def ensure_dir(self, path):
        if not exists(path):
            try:
                os.makedirs(path)
            except OSError as err:
                # FILE ALREADY EXISTS = 17
                if err.errno != 17:
                    raise
    
    def __is_expired(self, path):
        if STORAGE_EXPIRATION_SECONDS is None:
            return False
        timediff = datetime.now() - datetime.fromtimestamp(getmtime(path))
        return timediff.total_seconds() > STORAGE_EXPIRATION_SECONDS


